/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Tree = function()
{
	var tree = {};
	
	tree.initPage = function()
	{
		IconStore.addDefaultIcons();
		
		IconStore.addIconSet('ExtensionPoint', 'extensionpoint.png', 'extensionpoint-selected.png');
		IconStore.addIconSet('ExtensionPointDisabled', 'extensionpoint-disabled.png', 'extensionpoint-disabled-selected.png');
		IconStore.addIconSet('ExtensionPointError', 'extensionpoint-error.png', 'extensionpoint-error-selected.png');
		
		IconStore.addIconSet('Extension', 'extension.png', 'extension-selected.png');
		IconStore.addIconSet('ExtensionDisabled', 'extension-disabled.png', 'extension-disabled-selected.png');
		IconStore.addIconSet('ExtensionError', 'extension-error.png', 'extension-error-selected.png');
		IconStore.addIconSet('ExtensionWarning', 'extension-warning.png', 'extension-warning-selected.png');

		IconStore.addIconSet('Plugin', 'plugin.png', 'plugin-selected.png');
		IconStore.addIconSet('PluginDisabled', 'plugin-disabled.png', 'plugin-disabled-selected.png');
		IconStore.addIconSet('PluginError', 'plugin-error.png', 'plugin-error-selected.png');

		IconStore.addIconSet('JarFile', 'jarfile.png', 'jarfile-selected.png');
		IconStore.addIconSet('JarFileError', 'jarfile-error.png', 'jarfile-error-selected.png');
		IconStore.addIconSet('JarFileWarning', 'jarfile-warning.png', 'jarfile-warning-selected.png');
		IconStore.addIconSet('JarFileModified', 'jarfile-modified.png', 'jarfile-modified-selected.png');
		IconStore.addIconSet('JarFileModifiedError', 'jarfile-modified-error.png', 'jarfile-modified-error-selected.png');
		IconStore.addIconSet('JarFileModifiedWarning', 'jarfile-modified-warning.png', 'jarfile-modified-warning-selected.png');

		IconStore.addIconSet('XmlFile', 'item.png', 'item-selected.png');
		IconStore.addIconSet('XmlFileError', 'item-error.png', 'item-error-selected.png');
		IconStore.addIconSet('XmlFileWarning', 'item-warning.png', 'item-warning-selected.png');
		IconStore.addIconSet('XmlFileModified', 'item-modified.png', 'item-modified-selected.png');
		IconStore.addIconSet('XmlFileModifiedError', 'item-modified-error.png', 'item-modified-error-selected.png');
		IconStore.addIconSet('XmlFileModifiedWarning', 'item-modified-warning.png', 'item-modified-warning-selected.png');

		Events.addEventHandler('joust', 'joust-select', tree.onSelect);
		
		Joust2.draw('joust');
	}

	
	tree.onSelect = function(event)
	{
		var menuItem = event.target.item;
		
		if (menuItem.type == 'extension-point')
		{
			tree.extensionPointOnClick(menuItem.externalId);
		}
		else if (menuItem.type == 'extension')
		{
			tree.extensionOnClick(menuItem.externalId);
		}
		else if (menuItem.type == 'file')
		{
			tree.fileOnClick(menuItem.fileUri);
		}
		else if (menuItem.type == 'plugin')
		{
			tree.pluginOnClick(menuItem.className);
		}
		else if (menuItem.externalId.indexOf('root.') == 0)
		{
			tree.rootOnClick();
		}
	}
	
	tree.extensionPointOnClick = function(extensionPointId)
	{
		var url = 'details.jsp?ID='+App.getSessionId();
		url += '&extensionPointId='+encodeURIComponent(extensionPointId);
		parent.frames['details'].location.href = url;
	}
	
	tree.extensionOnClick = function(extensionId)
	{
		var url = 'details.jsp?ID='+App.getSessionId();
		url += '&extensionId='+encodeURIComponent(extensionId);
		parent.frames['details'].location.href = url;
	}
	
	tree.fileOnClick = function(fileUri)
	{
		var url = 'details.jsp?ID='+App.getSessionId();
		url += '&fileuri='+encodeURIComponent(fileUri);
		parent.frames['details'].location.href = url;
	}
	
	tree.pluginOnClick = function(className)
	{
		var url = '../plugindefinitions/index.jsp?ID='+App.getSessionId();
		url += '&cmd=ViewItemByClass&iframe=1';
		url += '&className='+encodeURIComponent(className);
		parent.frames['details'].location.href = url;
	}

	tree.rootOnClick = function()
	{
		var url = 'details.jsp?ID='+App.getSessionId();
		parent.frames['details'].location.href = url;
	}
	
	/**
		Select the node representing the extension with
		the given file name.
	*/
	tree.selectFile = function(filename)
	{
		var menuElement = Doc.element(filename);
		if (menuElement)
		{
			//Joust2.open(menuElement);
			Joust2.select(menuElement);
		}
	}
	
	/**
		Set the icon for all menu items with the given external id. 
		NOTE! A single extension may appear in multiple places in the tree!
	 */
	tree.setIcon = function(externalId, icon)
	{
		var icons = [];
		icons[externalId] = icon;
		tree.setIconsByExternalId(icons);
	}

	/**
		Enabled/disable all child icons for the selected
		menu item.
	*/
	tree.setChildIcons = function(menuElement, enable)
	{
		var menuElement = Doc.element(menuElement);
		if (!menuElement) return;
		
		var menuItem = menuElement.item;
		if (!menuItem.children) return;
		
		var icons = [];
		for (var childNo = 0; childNo < menuItem.children.length; childNo++)
		{
			var child = menuItem.children[childNo];
			
			if (child.type == 'group')
			{
				// Receusive call into grouped extensions
				tree.setChildIcons(child.id, enable);
			}
			else
			{
				var iconName = child.icon;
				if (iconName.indexOf('Error') == -1)
				{
					var isDisabled = iconName.indexOf('Disabled') > 0;
					if (enable && isDisabled)
					{
						iconName = iconName.replace('Disabled', '');
					}
					else if (!enable && !isDisabled)
					{
						iconName += 'Disabled';
					}
					icons[child.externalId] = iconName;
				}
			}
		}
		tree.setIconsByExternalId(icons);
	}

	
	/**
		'icons' is an array with <key>=<externalId> and <value>=<new icon name>
		Loop through all menu items and change the icon if the external id 
		matches.
	*/
	tree.setIconsByExternalId = function(icons)
	{
		var tree = Doc.element('joust');
		
		var menuElements = tree.getElementsByClassName('joustitem');
		for (var menuNo = 0; menuNo < menuElements.length; menuNo++)
		{
			var menuElement = menuElements[menuNo];
			var menuItem = menuElement.item;
			
			var iconName = icons[menuItem.externalId];
			if (iconName)
			{
				menuItem.icon = iconName;
				Joust2.updateIconsAndText(menuElement);
			}
		}
	}

	
	return tree;
}();

Doc.onLoad(Tree.initPage);
