/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Users = function()
{
	var users = {};
	var extendedProperties;
	
	/**
		Initialize the page.
	*/
	users.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', users.save);
			Buttons.addClickHandler('close', App.closeWindow);
				
			// Tab validation
			TabControl.addTabValidator('settings.info', users.validateUser);
			TabControl.addTabValidator('settings.extended', users.validateExtendedProperties);
			
			// Extended properties
			extendedProperties = Data.json('page-data', 'extended-properties');
			for (var i = 0; i < extendedProperties.length; i++)
			{
				var ep = extendedProperties[i];
				var valueType = ep.valueType;
				var fieldName = 'ep.'+ep.name;
				
				if (valueType == 'INT' || valueType == 'LONG')
				{
					// Block non-numeric keys from INT and LONG fields
					Events.addEventHandler(fieldName, 'keypress', Events.integerOnly);
				}
				else if (valueType == 'FLOAT' || valueType == 'DOUBLE')
				{
					// Block non-numeric keys from FLOAT and DOUBLE fields
					Events.addEventHandler(fieldName, 'keypress', Events.numberOnly);
				}
			}

		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'USER', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnClone', users.cloneUserOnClick, attributes);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'USER'};
			var tableAttributes = {'table-id': 'users'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnDefaultMembership', users.defaultMembershipOnClick);
			Buttons.addClickHandler('btnNewGroup', users.newGroupOnClick);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	// Add event handlers to the 'clone' and 'impersonate' icons
	users.initElements = function(element, autoInit)
	{
		if (autoInit == 'clone')
		{
			Buttons.addClickHandler(element, users.cloneUserOnClick);
		}
		else if (autoInit == 'impersonate')
		{
			Buttons.addClickHandler(element, users.impersonateUserOnClick);
		}
	}
	
	users.validateUser = function()
	{
		var frm = document.forms['user'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name for the user.');
			return false;
		}
		if (Strings.trim(frm.login.value) == '')
		{
			Forms.showNotification(frm.login, 'You must enter a login for the user.');
			return false;
		}
		if (frm.new_password.value != frm.retype_password.value)
		{
			Forms.showNotification(frm.retype_password, 'The two passwords do not match!');
			return false;
		}
		
		if (frm.item_id.value == 0)
		{
			if (Strings.trim(frm.new_password.value) == '')
			{
				Forms.showNotification(frm.new_password, 'You must enter a password for the user.');
				return false;
			}
		}
		return true;
	}
	
	
	// Validate the 'Other information' tab with extended properties
	users.validateExtendedProperties = function()
	{
		var frm = document.forms['user'];
		var dateFormat = Data.get('page-data', 'date-format');
		var dateTimeFormat = Data.get('page-data', 'datetime-format');
		
		for (var i = 0; i < extendedProperties.length; i++)
		{
			var ep = extendedProperties[i];
			var valueType = ep.valueType;
			var field = frm['ep.'+ep.name];
			
			if (ep.valueType == 'DATE')
			{
				if (field.value != '' && !Dates.isDate(field.value, dateFormat))
				{
					Forms.showNotification(field, "'"+field.value+"' is not a valid date for '" + ep.title + "'");
					return false;
				}
			}
			else if (ep.valueType == 'TIMESTAMP')
			{
				if (field.value != '' && !Dates.isDate(field.value, dateTimeFormat))
				{
					Forms.showNotification(field, "'"+field.value+"' is not a valid timestamp for '" + ep.title + "'");
					return false;
				}
			}
			if (!ep.nullable)
			{
				if (ep.valueType == 'BOOLEAN')
				{
					if (Forms.getCheckedRadio(field) == null)
					{
						Forms.showNotification(field[0], "You must select a value for '" + ep.title + "'");
						return false;
					}
				}
				else
				{
					if (Strings.trim(field.value) == '')
					{
						Forms.showNotification(field, "You must enter a value for '" + ep.title + "'");
						return false;
					}
				}
			}
		}	
		return true;
	}

	
	users.save = function()
	{
		var frm = document.forms['user'];
		if (TabControl.validateActiveTab('settings'))
		{
			Link.exportActions('membership');
			frm.submit();
		}
	}
	
	// Open the 'Default membership' dialog
	users.defaultMembershipOnClick = function()
	{
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=EditDefaultMembership';
		Dialogs.openPopup(url, 'DefaultMembership', 600, 400);
	}
	
	// Create a new group item with the selected users as members
	users.newGroupOnClick = function()
	{
		var selected = Table.getSelected('users', null, 'Please select at least one item in the list');
		if (selected.length == 0) return;

		var extraUrl = '&user_id='+selected.join('&user_id=');
		Items.newItem('GROUP', extraUrl);
	}
	
	// Click handler for cloning a user
	users.cloneUserOnClick = function(event)
	{
		var target = event.currentTarget;
		var userId = Data.get(target, 'item-id');
		Items.newItem('USER', '&clone_id='+userId);
	}
	
	// Click handler for opening the 'Impersonate login' dialog
	users.impersonateUserOnClick = function(event)
	{
		var target = event.currentTarget;
		var userId = Data.get(target, 'item-id');
		var url = App.getRoot()+'impersonate.jsp?ID='+App.getSessionId();
		url += '&user_id='+userId;
		Dialogs.openPopup(url, 'Impersonate', 450, 300);
	}
	
	return users;
}();

Doc.addElementInitializer(Users.initElements);
Doc.onLoad(Users.initPage);
