/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var JepExtraValueCalculator = function()
{
	var jep = {};
	
	/**
		Initialize the page.
	*/
	jep.initPage = function()
	{
		// BioAssays
		Buttons.addClickHandler('btnAddBioAssays', jep.addBioAssays);
		Events.addEventHandler('allBioAssays', 'click', jep.allBioAssaysOnClick);
		
		var frm = document.forms['extraValue'];
		frm.allBioAssays.checked = frm.bioAssays.length == 0;
		jep.allBioAssaysOnClick();
		
		// Extra value type
		Buttons.addClickHandler('parameter:extraValueType.select', jep.selectExtraValueType);
		Events.addEventHandler('parameter:extraValueType', 'base-selected', Items.onItemSelected);
		
		// Preset formulas
		Events.addEventHandler('presets', 'change', JepPlugins.presetOnChange);
		Buttons.addClickHandler('btnExpressionBuilder', JepPlugins.openExpressionBuilder);
		
		Buttons.addClickHandler('btnNext', jep.goNext);
	}
	
	jep.allBioAssaysOnClick = function()
	{
		var frm = document.forms['extraValue'];
		frm.bioAssays.disabled = frm.allBioAssays.checked;
	}

	jep.addBioAssays = function()
	{
		var frm = document.forms['extraValue'];
		frm.allBioAssays.checked = false;
		frm.bioAssays.disabled = false;
		var ids = Link.getIdsInList(frm.bioAssays, 'BIOASSAY');
		var url = '&resetTemporary=1&exclude='+ids.join(',');
		url += '&bioassayset_id='+frm['parameter:source'].value;
		Dialogs.selectItem('BIOASSAY', 'bioAssays', 1, url);
	}
	
	jep.selectExtraValueType = function()
	{
		var frm = document.forms['extraValue'];
		var url = '&resetTemporary=1';
		if (frm['parameter:extraValueType'].length > 1) 
		{
			var id = Math.abs(parseInt(frm['parameter:extraValueType'][1].value));
			url += '&item_id='+id;
		}
		Dialogs.selectItem('EXTRAVALUETYPE', 'parameter:extraValueType', 0, url);
	}
	
	jep.validateParameters = function()
	{
		var frm = document.forms['extraValue'];
		if (!frm.allBioAssays.checked && frm.bioAssays.length == 0)
		{
			Forms.showNotification('btnAddBioAssays', 'You have not selected any bioassays');
			return false;
		}
		else if (frm['parameter:extraValueType'].selectedIndex < 0)
		{
			Forms.showNotification('parameter:extraValueType.select', 'You must select a type');
			return false;
		}
		else if (Strings.trim(frm['parameter:expression'].value) == '')
		{
			Forms.showNotification(frm['parameter:expression'], 'You must enter an expression for the extra value');
			return false;
		}
		if (Data.int('page-data', 'max-raw-mapping') != 1)
		{
			if (frm['parameter:expression'].value.search(/raw\(/) != -1)
			{
				Forms.showNotification(frm['parameter:expression'], 'Can\'t use function raw() for bioassaysets with multiple mappings to raw data');
				return false;
			}
		}
		return true;
	}
	
	jep.goNext = function()
	{
		if (!jep.validateParameters()) return;

		var frm = document.forms['extraValue'];
		if (!frm.allBioAssays.checked)
		{
			var ids = Link.getIdsInList(frm.bioAssays, 'BIOASSAY');
			for (var i = 0; i < ids.length; i++)
			{
				Forms.addHidden(frm, 'parameter:bioAssays', ids[i]);
			}
		}
		frm.submit();
	}
	
	jep.presetOnChange = function(event)
	{
		var formula = Doc.element(Data.get(event.currentTarget, 'formula'));
		formula.value = event.currentTarget.value;
	}

	return jep;
}();

Doc.onLoad(JepExtraValueCalculator.initPage);

